@@ -6,7 +6,7 @@ import os
 import shutil
 from collections.abc import Container, Iterable, Iterator, Sequence
 from contextlib import suppress
-from typing import Any, NamedTuple, TypedDict
+from typing import Any, Callable, NamedTuple, TypedDict
 
 from .boss import Boss
 from .child import Child
@@ -16,7 +16,7 @@ from .clipboard import set_clipboard_string, set_primary_selection
 from .fast_data_types import add_timer, get_boss, get_options, get_os_window_title, patch_color_profiles
 from .options.utils import env as parse_env
 from .tabs import Tab, TabManager
-from .types import OverlayType, run_once
+from .types import LayerShellConfig, OverlayType, run_once
 from .utils import get_editor, log_error, resolve_custom_file, which
 from .window import CwdRequest, CwdRequestType, Watchers, Window
 
@@ -81,7 +81,7 @@ of the active window in the tab is used as the tab title. The special value
 --type
 type=choices
 default=window
-choices=window,tab,os-window,overlay,overlay-main,background,clipboard,primary
+choices=window,tab,os-window,os-panel,overlay,overlay-main,background,clipboard,primary
 Where to launch the child process:
 
 :code:`window`
@@ -116,6 +116,11 @@ Where to launch the child process:
     These two are meant to work with :option:`--stdin-source <launch --stdin-source>` to copy
     data to the :italic:`system clipboard` or :italic:`primary selection`.
 
+:code:`os-panel`
+    Similar to :code:`os-window`, except that it creates the new OS Window as a desktop panel.
+    Only works on platforms that support this, such as Wayand compositors that support the layer
+    shell protocol. Use the :option:`kitten @ launch --os-panel` option to configure the panel.
+
 #placeholder_for_formatting#
 
 
@@ -151,6 +156,7 @@ times to set different user variables. Syntax: :code:`name=value`. Using
 --hold
 type=bool-set
 Keep the window open even after the command being executed exits, at a shell prompt.
+The shell will be run after the launched command exits.
 
 
 --copy-colors
@@ -377,6 +383,15 @@ the section on watchers in the launch command documentation: :ref:`watchers`.
 Relative paths are resolved relative to the :ref:`kitty config directory
 <confloc>`. Global watchers for all windows can be specified with
 :opt:`watcher` in :file:`kitty.conf`.
+
+
+--os-panel
+type=list
+Options to control the creation of desktop panels. Takes the same settings
+as the :doc:`panel kitten </kittens/panel>`. Can be specified multiple times.
+For example, to create a desktop panel at the bottom of the screen two lines high::
+
+    launch --type os-panel --os-panel lines=2 --os-panel edge=bottom sh -c "echo; echo; echo hello; sleep 5s"
 """
 
 
@@ -401,15 +416,28 @@ def get_env(opts: LaunchCLIOptions, active_child: Child | None = None, base_env:
     return env
 
 
+def layer_shell_config_from_panel_opts(panel_opts: Iterable[str]) -> LayerShellConfig:
+    from kittens.panel.main import layer_shell_config, parse_panel_args
+    args = [('' if x.startswith('--') else '--') + x for x in panel_opts]
+    try:
+        opts, _ = parse_panel_args(args)
+    except SystemExit as e:
+        raise ValueError(str(e))
+    return layer_shell_config(opts)
+
+
 def tab_for_window(boss: Boss, opts: LaunchCLIOptions, target_tab: Tab | None, next_to: Window | None) -> Tab:
 
     def create_tab(tm: TabManager | None = None) -> Tab:
         if tm is None:
-            oswid = boss.add_os_window(
-                wclass=opts.os_window_class,
-                wname=opts.os_window_name,
-                window_state=opts.os_window_state,
-                override_title=opts.os_window_title or None)
+            if opts.type == 'os-panel':
+                oswid = boss.add_os_panel(layer_shell_config_from_panel_opts(opts.os_panel), opts.os_window_class, opts.os_window_name)
+            else:
+                oswid = boss.add_os_window(
+                    wclass=opts.os_window_class,
+                    wname=opts.os_window_name,
+                    window_state=opts.os_window_state,
+                    override_title=opts.os_window_title or None)
             tm = boss.os_window_map[oswid]
         tab = tm.new_tab(empty_tab=True, location=opts.location)
         if opts.tab_title:
@@ -423,7 +451,7 @@ def tab_for_window(boss: Boss, opts: LaunchCLIOptions, target_tab: Tab | None, n
         if target_tab is not None:
             tm = target_tab.tab_manager_ref() or tm
         tab = create_tab(tm)
-    elif opts.type == 'os-window':
+    elif opts.type in ('os-window', 'os-panel'):
         tab = create_tab()
     else:
         if target_tab is not None:
@@ -565,6 +593,7 @@ def _launch(
     is_clone_launch: str = '',
     rc_from_window: Window | None = None,
     base_env: dict[str, str] | None = None,
+    child_death_callback: Callable[[int, Exception | None], None] | None = None,
 ) -> Window | None:
     source_window = boss.active_window_for_cwd
     if opts.source_window:
@@ -702,7 +731,8 @@ def _launch(
             raise ValueError('The cmd to run must be specified when running a background process')
         boss.run_background_process(
             cmd, cwd=kw['cwd'], cwd_from=kw['cwd_from'], env=env or None, stdin=kw['stdin'],
-            allow_remote_control=kw['allow_remote_control'], remote_control_passwords=kw['remote_control_passwords']
+            allow_remote_control=kw['allow_remote_control'], remote_control_passwords=kw['remote_control_passwords'],
+            notify_on_death=child_death_callback,
         )
     elif opts.type in ('clipboard', 'primary'):
         stdin = kw.get('stdin')
@@ -713,6 +743,8 @@ def _launch(
             else:
                 set_primary_selection(stdin)
                 boss.handle_clipboard_loss('primary')
+        if child_death_callback is not None:
+            child_death_callback(0, None)
     else:
         kw['hold'] = opts.hold
         if force_target_tab and target_tab is not None:
@@ -723,6 +755,8 @@ def _launch(
         with Window.set_ignore_focus_changes_for_new_windows(opts.keep_focus):
             new_window: Window = tab.new_window(
                 env=env or None, watchers=watchers or None, is_clone_launch=is_clone_launch, next_to=next_to, **kw)
+            if child_death_callback is not None:
+                boss.monitor_pid(new_window.child.pid or 0, child_death_callback)
         if spacing:
             patch_window_edges(new_window, spacing)
             tab.relayout()
@@ -753,12 +787,13 @@ def launch(
     is_clone_launch: str = '',
     rc_from_window: Window | None = None,
     base_env: dict[str, str] | None = None,
+    child_death_callback: Callable[[int, Exception | None], None] | None = None,
 ) -> Window | None:
     active = boss.active_window
     if opts.keep_focus and active:
         orig, active.ignore_focus_changes = active.ignore_focus_changes, True
     try:
-        return _launch(boss, opts, args, target_tab, force_target_tab, is_clone_launch, rc_from_window, base_env)
+        return _launch(boss, opts, args, target_tab, force_target_tab, is_clone_launch, rc_from_window, base_env, child_death_callback)
     finally:
         if opts.keep_focus and active:
             active.ignore_focus_changes = orig
